'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _Memoize = require('./Memoize');

var _Memoize2 = _interopRequireDefault(_Memoize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
  primaryAxes: function primaryAxes() {
    return (0, _Memoize2.default)(function (state) {
      return [state.axes];
    }, function (axes) {
      if (!axes) return [];
      return Object.keys(axes).map(function (key) {
        if (axes[key].primary) {
          return axes[key];
        }
        return null;
      }).filter(Boolean);
    });
  },

  secondaryAxes: function secondaryAxes() {
    return (0, _Memoize2.default)(function (state) {
      return [state.axes];
    }, function (axes) {
      if (!axes) return [];
      return Object.keys(axes).map(function (key) {
        if (!axes[key].primary) {
          return axes[key];
        }
        return null;
      }).filter(Boolean);
    });
  },

  offset: function offset() {
    return (0, _Memoize2.default)(function (state) {
      return [state && state.offset && state.offset.left || 0, state && state.offset && state.offset.top || 0];
    }, function (left, top) {
      return {
        left: left,
        top: top
      };
    });
  },

  gridX: function gridX() {
    return (0, _Memoize2.default)(function (state) {
      return [state && state.padding && state.padding.left || 0, state && state.axisDimensions && state.axisDimensions.left && sumObjBy(state.axisDimensions.left, 'width') || 0, state && state.axisDimensions && state.axisDimensions.top && sumObjBy(state.axisDimensions.top, 'left') || 0, state && state.axisDimensions && state.axisDimensions.bottom && sumObjBy(state.axisDimensions.bottom, 'left') || 0];
    }, function (paddingLeft, axesLeftWidth, axesTopLeft, axesBottomLeft) {
      return paddingLeft + Math.max(axesLeftWidth, axesTopLeft, axesBottomLeft);
    });
  },

  gridY: function gridY() {
    return (0, _Memoize2.default)(function (state) {
      return [state && state.padding && state.padding.top || 0, state && state.axisDimensions && state.axisDimensions.top && sumObjBy(state.axisDimensions.top, 'height') || 0, state && state.axisDimensions && state.axisDimensions.left && sumObjBy(state.axisDimensions.left, 'top') || 0, state && state.axisDimensions && state.axisDimensions.right && sumObjBy(state.axisDimensions.right, 'top') || 0];
    }, function (paddingTop, axesTopHeight, axesLeftTop, axesRightTop) {
      return paddingTop + Math.max(axesTopHeight, axesLeftTop, axesRightTop);
    });
  },

  gridWidth: function gridWidth() {
    return (0, _Memoize2.default)(function (state) {
      return [state && state.width && state.width || 0, state && state.padding && state.padding.left || 0, state && state.padding && state.padding.right || 0, state && state.axisDimensions && state.axisDimensions.left && sumObjBy(state.axisDimensions.left, 'width') || 0, state && state.axisDimensions && state.axisDimensions.right && sumObjBy(state.axisDimensions.right, 'width') || 0, state && state.axisDimensions && state.axisDimensions.top && sumObjBy(state.axisDimensions.top, 'left') || 0, state && state.axisDimensions && state.axisDimensions.top && sumObjBy(state.axisDimensions.top, 'right') || 0, state && state.axisDimensions && state.axisDimensions.bottom && sumObjBy(state.axisDimensions.bottom, 'left') || 0, state && state.axisDimensions && state.axisDimensions.bottom && sumObjBy(state.axisDimensions.bottom, 'right') || 0];
    }, function (width, paddingLeft, paddingRight, axesLeftWidth, axesRightWidth, axesTopLeft, axesTopRight, axesBottomLeft, axesBottomRight) {
      return width - paddingLeft - paddingRight - Math.max(axesLeftWidth, axesTopLeft, axesBottomLeft) - Math.max(axesRightWidth, axesTopRight, axesBottomRight);
    });
  },

  gridHeight: function gridHeight() {
    return (0, _Memoize2.default)(function (state) {
      return [state && state.height || 0, state && state.padding && state.padding.top || 0, state && state.padding && state.padding.bottom || 0, state && state.axisDimensions && state.axisDimensions.top && sumObjBy(state.axisDimensions.top, 'height') || 0, state && state.axisDimensions && state.axisDimensions.bottom && sumObjBy(state.axisDimensions.bottom, 'height') || 0, state && state.axisDimensions && state.axisDimensions.left && sumObjBy(state.axisDimensions.left, 'top') || 0, state && state.axisDimensions && state.axisDimensions.left && sumObjBy(state.axisDimensions.left, 'bottom') || 0, state && state.axisDimensions && state.axisDimensions.right && sumObjBy(state.axisDimensions.right, 'top') || 0, state && state.axisDimensions && state.axisDimensions.right && sumObjBy(state.axisDimensions.right, 'bottom') || 0];
    }, function (height, paddingTop, paddingBottom, axesTopHeight, axesBottomHeight, axesLeftTop, axesLeftBottom, axesRightTop, axesRightBottom) {
      return height - paddingTop - paddingBottom - Math.max(axesTopHeight, axesLeftTop, axesRightTop) - Math.max(axesBottomHeight, axesLeftBottom, axesRightBottom);
    });
  }
};


function sumObjBy(obj, str) {
  return Object.keys(obj).map(function (key) {
    return obj[key];
  }).reduce(function (prev, curr) {
    return prev + curr[str] || 0;
  }, 0);
}